/*	Analog.cpp
*
*	Analog Input using Analog to Digital Converter (ADC).
*
*	Written by Cathy Saxton
*	robotics@idleloop.com
*/

#include <avr/io.h>
#include "main.h"

bool ANALOG::s_fInit = false;

ANALOG::ANALOG(char channel)
{
	m_channel = channel;
	
	if (!s_fInit)
		Init();
}

/* sets up the Analog-to-Digital Converter (ADC) */
void ANALOG::Init()
{
	s_fInit = true;
	
	/* we left-adjust so that we get the 8 most-significant bits
	   in ADCH; use VCC (5V) for reference voltage;
	   prescaler=64 for 128kHz ADC clock (from 8MHz uC clk) */
	ADMUX = 0x60;	// 01 1 0 0000 (Aref, L-shift, [ADC0])
	ADCSRA = 0x86;	// 1000 0 110 (enable, no auto trigger, no interrupt, clk/64)
}

/* reads the specified ADC channel (0-7) and returns its value (0-255) */
unsigned char ANALOG::ReadByte()
{
	ADMUX &= ~0x1F;			// clear channel selection (low 5 bits)
	ADMUX |= m_channel;		// select specified channel

	SetBit(ADCSRA, ADSC);	// ADC start conversion
		/* (this appears to clear ADIF) */

	/* wait for conversion to complete */
	while (bit_is_clear(ADCSRA, ADIF))
		;

	return (unsigned char) ADCH;
}
