/*	InOut.h
*
*	Declarations for classes that handle input and output.
*
*	Written by Cathy Saxton
*	robotics@idleloop.com
*/

#pragma once

/* IN -- input class, e.g. for switches */
class IN
{
public:
	/* constructor for setting the register ('A', 'B', etc) and
	   pin (0-7), and specifying whether to enable the pull-up resistor */
	IN(char reg, char pin, bool fPullup);
	
	/* returns true if pin is reading high, else false */
	bool IsHigh() const;
	
	/* returns true if pin is reading low, else false */
	inline bool IsLow() const { return !IsHigh(); };
	
	/* helpers for switches */
	inline bool IsOpen() const   { return IsLow(); }
	inline bool IsClosed() const { return IsHigh();  }
	
	/* checks for button press; if not pressed, returns false; else
	   waits for release (including debouncing both press & release) */
	bool WasPressed() const;
	
	/* specify whether to beep when button is pressed in WasPressed */
	inline void SetBeep(bool fBeep)	{ m_fBeep = fBeep; }

private:
	bool m_fBeep;	// whether to beep in WasPressed
	unsigned char m_bit;
	volatile uint8_t *m_preg;
};

/* OUT -- output class, e.g. for LEDs */
class OUT
{
public:
	/* constructor for setting the register ('A', 'B', etc),
	   port (0-7), and initial state (low/high output) */
	OUT(char reg, char port, bool fStartHigh);
	
	void SetHigh() const;	// set output high
	void SetLow() const;	// set output low
	void Toggle() const;	// toggle output
	
	/* helpers for LEDs */
	inline void On() const  { SetHigh(); };
	inline void Off() const { SetLow(); };
	void Set(char fOn) const;	// set LED on or off
	
private:
	unsigned char m_bit;
	volatile uint8_t *m_preg;
	friend class MOTOR;
};
